@@ -43,6 +43,19 @@ def val_err_str(val: float, err: float) -> str:
     return f"{val:.{max(0, dps)}f}({err:.0f})"
 
 
+def shrink_path_for_display(filename: str, max_len: int = 63) -> str:
+    path_str = os.path.abspath(filename)
+    if len(path_str) < max_len:
+        return path_str
+    if max_len < 5:
+        raise ValueError(f"{max_len} is too short. Try a sensible value, like 43 or something...")
+    max_len -= 3
+    max_len //= 2
+    path_left = path_str[:max_len]
+    path_right = path_str[-max_len:]
+    return f"{path_left}...{path_right}"
+
+
 class DiffractionDataPoint:
     """
     This is a data point in a diffraction patter.
@@ -64,7 +77,7 @@ class DiffractionDataPoint:
     def zeroOffset(self, offset: float):
         self.x += offset
 
-    def average_with(self, ddps: List[DiffractionDataPoint], in_place: bool = True) -> DiffractionDataPoint:
+    def average_with(self, ddps: List[DiffractionDataPoint], in_place: bool = True, sum_instead: bool = False) -> DiffractionDataPoint:
         if not ddps:  # empty list
             return copy.deepcopy(self)
 
@@ -75,12 +88,13 @@ class DiffractionDataPoint:
                 raise ValueError(f"trying to average data at {ddp.x} with data at {self.x}.")
             ddp_mean.increment(ddp)
 
+        r = ddp_mean.get_result() if not sum_instead else ddp_mean.get_result() * ddp_mean.n
         if in_place:
-            self.x = ddp_mean.get_result().x
-            self.y = ddp_mean.get_result().y
-            self.e = ddp_mean.get_result().e
+            self.x = r.x
+            self.y = r.y
+            self.e = r.e
         else:
-            return ddp_mean.get_result()
+            return r
 
     def __str__(self) -> str:
         return f"{self.x:.5f} {self.y:.3f} {self.e:.3f}"
@@ -452,7 +466,7 @@ class DiffractionPattern:
     """
     DROP_POINTS_IN_SPLINE = 5
 
-    def __init__(self, diffpat: List[DiffractionDataPoint] = None, filename: str = None, meta=None):
+    def __init__(self, diffpat: List[DiffractionDataPoint] = None, filename: str = None, meta: dict = None):
         """
         Read in inital data in order to make everything.
         Angles must be strictly increasing
@@ -461,15 +475,16 @@ class DiffractionPattern:
         ----------
         diffpat : a list of DiffractionDataPoints
         filename: a string representing the filename of the diffraction data
-        meta: any metainformation about the diffraction pattern.
+        meta: a dictionary containing metainformation about the diffraction pattern.
 
         Returns
         -------
         None.
         """
-        self.filename = filename
+        self.filenamebase = os.path.splitext(filename)[0] or ""
+        self.filename = filename or ""
         self.diffpat = Read.read(filename) if diffpat is None else diffpat
-        self.meta = meta  # any information at all about this diffraction pattern. Can be anything of any type.
+        self.meta = meta or {}
 
         self.xs = [ddp.x for ddp in self.diffpat]
         self.ys = [ddp.y for ddp in self.diffpat]
@@ -481,19 +496,17 @@ class DiffractionPattern:
     def negate(self):
         for d in self.diffpat:
             d.negate()
+        self.meta["negate"] = True if "negate" not in self.meta else not self.meta["negate"]
 
     def reverse(self):
         self.diffpat.reverse()
+        self.meta["reverse"] = True if "reverse" not in self.meta else not self.meta["reverse"]
 
-    def zero_offset(self, offset: float | List[float]):
+    def zero_offset(self, offset: float):
         if isinstance(offset, float):
             for d in self.diffpat:
                 d.zeroOffset(offset)
-        elif isinstance(offset, list):
-            if len(offset) != len(self.diffpat):
-                raise ValueError(f"Need {len(self.diffpat)} values. Was given {len(offset)} instead.")
-            for d, o in zip(self.diffpat, offset):
-                d.zeroOffset(o)
+        self.meta["zero_offset"] = offset
 
     def __len__(self) -> int:
         return len(self.diffpat)
@@ -512,6 +525,7 @@ class DiffractionPattern:
         """
         Trims a diffraction pattern such that there exist no angles less
         than min_angle, and no angles greater than max_angle.
+        ie min_x <= angles <= max_x
 
         Parameters
         ----------
@@ -521,12 +535,17 @@ class DiffractionPattern:
             the maximum angle you want to see in your diffraction pattern. The default is 180.
         in_place: bool, alter self, or return a new Diffractionpattern?
         """
-        dp = [xdp for xdp in self.diffpat if min_x <= xdp.x <= max_x]
+        dp = [ddp for ddp in self.diffpat if min_x <= ddp.x <= max_x]
 
         if in_place:
-            self.diffpat[:] = dp
+            if len(dp) < len(self):
+                self.diffpat[:] = dp
+                self.meta["trim"] = True
         else:
-            return DiffractionPattern(diffpat=dp)
+            meta = copy.deepcopy(self.meta)
+            if len(dp) < len(self):
+                meta["trim"] = True
+            return DiffractionPattern(diffpat=dp, filename=self.filename, meta=meta)
 
     def sort(self, in_place: bool = True):
         """
@@ -534,10 +553,13 @@ class DiffractionPattern:
         """
         if in_place:
             self.diffpat.sort()
+            self.meta["sort"] = True
         else:
             dp = copy.deepcopy(self.diffpat)
             dp.sort()
-            return DiffractionPattern(diffpat=dp)
+            meta = copy.deepcopy(self.meta)
+            meta["sort"] = True
+            return DiffractionPattern(diffpat=dp, filename=self.filename, meta=meta)
 
     def downsample(self, ds: int, in_place: bool = True) -> Optional[DiffractionPattern]:
         """
@@ -564,8 +586,11 @@ class DiffractionPattern:
 
         if in_place:
             self.diffpat[:] = r
+            self.meta["downsample"] = ds
         else:
-            return DiffractionPattern(r)
+            meta = copy.deepcopy(self.meta)
+            meta["downsample"] = ds
+            return DiffractionPattern(r, filename=self.filename, meta=meta)
 
     def split_on_zero(self) -> tuple[DiffractionPattern | None, DiffractionPattern | None]:
         """
@@ -579,8 +604,8 @@ class DiffractionPattern:
         max_ = self.diffpat[-1].x
         do_pos = False  # which angles do I do? positive? negative?
         do_neg = False
-        pos = None
-        neg = None
+        pos: DiffractionPattern = None
+        neg: DiffractionPattern = None
 
         if min_ < 0 and max_ > 0:
             do_pos = True
@@ -593,21 +618,21 @@ class DiffractionPattern:
             raise ValueError("Your data isn't monotonically increasing")
 
         if do_pos:
-            pos = copy.deepcopy(self)
-            pos.trim(0.0001, 180)
+            pos = self.trim(0.0001, 180, in_place=False)
+            pos.meta["split_on_zero"] = True
         if do_neg:
-            neg = copy.deepcopy(self)
-            neg.trim(-180, -0.0001)
+            neg = self.trim(-180, -0.0001, in_place=False)
             neg.negate()
             neg.reverse()
+            neg.meta["split_on_zero"] = True
 
         return pos, neg
 
-    def average_with(self, dps: List[DiffractionPattern], in_place: bool = True) -> Optional[DiffractionPattern]:
+    def average_with(self, other_dps: List[DiffractionPattern], in_place: bool = True, sum_instead: bool = False) -> Optional[DiffractionPattern]:
         # get deepcopy of the diffpats so I'm not plagued by pointer errors
 
         src = self.diffpat if in_place else copy.deepcopy(self.diffpat)
-        for dp in dps:
+        for dp in other_dps:
             src += copy.deepcopy(dp.diffpat)
         src.sort()
 
@@ -624,14 +649,17 @@ class DiffractionPattern:
                 elif p2 == p1:
                     tmp.append(p2)
                     j += 1
-            dest.append(p1.average_with(tmp, in_place=False))
+            dest.append(p1.average_with(tmp, in_place=False, sum_instead=sum_instead))
             tmp = []
             i = j
 
         if in_place:
             self.diffpat[:] = dest
+            self.meta["average_with"] = len(other_dps)
         else:
-            return DiffractionPattern(diffpat=dest)
+            meta = copy.deepcopy(self.meta)
+            meta["average_with"] = len(other_dps)
+            return DiffractionPattern(diffpat=dest, filename=self.filename, meta=meta)
 
     def interpolate(self, step_size: float, in_place: bool = True) -> None | DiffractionPattern:
         DP = DiffractionPattern
@@ -672,8 +700,11 @@ class DiffractionPattern:
 
         if in_place:
             self.diffpat[:] = r
+            self.meta["interpolate"] = step_size
         else:
-            return DiffractionPattern(diffpat=r)
+            meta = copy.deepcopy(self.meta)
+            meta["interpolate"] = step_size
+            return DiffractionPattern(diffpat=r, filename=self.filename, meta=meta)
 
     @staticmethod
     def _generateInterpList(min_start: float, max_stop: float, step_size: float) -> List[float]:
@@ -858,6 +889,28 @@ class DiffractionPattern:
             f0 = f0[0]
         return f0
 
+    def normalised_intensities(self) -> DiffractionPattern:
+        """
+        Construct a diffraction pattern such that the errors of the intensities are given by the sqrt
+        of the intensities.
+
+        i/sqrt(i) = I/E
+        sqrt(i) = I/E
+        i = (I/E)**2
+
+        This is purely for display purposes, and shows higher intensities where the intensities are better known.
+
+        :return: a DiffractionPattern where errors are the sqrt of the intensities
+        """
+        diffpat = [
+            DiffractionDataPoint(ddp.x, (ddp.y / ddp.e) ** 2, ddp.y / ddp.e)
+            for ddp in self.diffpat
+        ]
+
+        meta = copy.deepcopy(self.meta)
+        meta["normalised_intensities"] = True
+        return DiffractionPattern(diffpat=diffpat, filename=self.filename, meta=meta)
+
     def _add_sub(self, other: DiffractionPattern, op: operator) -> DiffractionPattern:
         """
         This function is used by __add__ and __and__ to allow for + and &
@@ -1278,7 +1331,7 @@ class DiffractionExperiment:
     It is essentially a list of DiffractionPatterns.
     """
 
-    def __init__(self, diffpats: List[DiffractionPattern], filename: str = None, meta=None):
+    def __init__(self, diffpats: List[DiffractionPattern] = None, filenames: List[str] = None, meta: dict = None):
         """
         Read in inital data in order to make everything.
         Angles must be strictly increasing
@@ -1286,16 +1339,16 @@ class DiffractionExperiment:
         Parameters
         ----------
         diffpats : an array of DiffractionPatterns
-        filename : a filename of a file that contains many diffraction patterns
+        filename : list of filenames
         meta: any information at all you wish to associate with the DiffractionExperiment
 
         Returns
         -------
         None.
         """
-        self.filename = filename
-        self.diffpats = diffpats
-        self.meta = meta  # any information at all about this diffraction pattern. Can be anything of any type.
+        self.filenames = [dp.filename for dp in diffpats] if diffpats else filenames
+        self.diffpats = [DiffractionPattern(filename=filename) for filename in filenames] if filenames else diffpats
+        self.meta = meta or {}
 
     def negate(self):
         for d in self.diffpats:
@@ -1353,7 +1406,7 @@ class DiffractionExperiment:
                 dp.trim(min_val, max_val)
         else:
             dps = [dp.trim(min_x, max_x, in_place=False) for dp, min_x, max_x in zip(self.diffpats, min_x, max_x)]
-            return DiffractionExperiment(diffpats=dps)
+            return DiffractionExperiment(diffpats=dps, filenames=self.filenames, meta=copy.deepcopy(self.meta))
 
     def sort(self, in_place: bool = True) -> None | DiffractionExperiment:
         """
@@ -1364,7 +1417,7 @@ class DiffractionExperiment:
                 dp.sort()
         else:
             dps = [dp.sort(in_place=False) for dp in self.diffpats]
-            return DiffractionExperiment(diffpats=dps)
+            return DiffractionExperiment(diffpats=dps, filenames=self.filenames, meta=copy.deepcopy(self.meta))
 
     def downsample(self, ds: int, in_place: bool = True) -> None | DiffractionExperiment:
         """
@@ -1380,33 +1433,38 @@ class DiffractionExperiment:
                 dp.downsample(ds)
         else:
             dps = [dp.downsample(ds, in_place=False) for dp in self.diffpats]
-            return DiffractionExperiment(diffpats=dps)
+            return DiffractionExperiment(diffpats=dps, filenames=self.filenames, meta=copy.deepcopy(self.meta))
 
-    def average_patterns(self, num: int, is_rolling: bool = True, in_place: bool = True) -> None | DiffractionExperiment:
+    def average_patterns(self, num: int = None, is_rolling: bool = True, in_place: bool = True, sum_instead: bool = False) -> None | DiffractionExperiment:
         """
         Averages num patterns. If rolling average, then
         patterns 1..num are averaged, then 2..num+1, 3--num+2 and so on.
         If not rolling average, then 1..num, num+1..num+num and so on.
-        :param num: number of patterns to average
+        :param num: number of patterns to average. Defaults to averageing all patterns
         :param is_rolling: do a rolling average?
         :return:
         """
+        if num is None:
+            num = len(self)
         if num <= 1:
             raise ValueError("You need to average more than 1 dataset together.")
         if num > len(self):
             raise ValueError(f"You tried to average {num} patterns where there is only {len(self)} to choose from.")
 
         range_step = 1 if is_rolling else num
-        dps = []
-        for i in range(0, len(self.diffpats) - num + 1, range_step):
+        dps: List[DiffractionPattern] = []
+        for i in range(0, len(self) - num + 1, range_step):
             dp = copy.deepcopy(self.diffpats[i])
             tmp = [self.diffpats[i + j] for j in range(1, num)]
-            dps.append(dp.average_with(tmp, in_place=False))
+            ave_num = i / num
+            dp.average_with(tmp, in_place=True, sum_instead=sum_instead)
+            dp.meta["average_pattern"] = ave_num
+            dps.append(dp)
 
         if in_place:
             self.diffpats[:] = dps
         else:
-            return DiffractionExperiment(diffpats=dps)
+            return DiffractionExperiment(diffpats=dps, filenames=self.filenames, meta=copy.deepcopy(self.meta))
 
     def interpolate(self, step: float, in_place: bool = True) -> None | DiffractionExperiment:
         if in_place:
@@ -1414,7 +1472,7 @@ class DiffractionExperiment:
                 dp.interpolate(step, in_place=True)
         else:
             idps = [dp.interpolate(step, in_place=False) for dp in self.diffpats]
-            return DiffractionExperiment(diffpats=idps)
+            return DiffractionExperiment(diffpats=idps, filenames=self.filenames, meta=copy.deepcopy(self.meta))
 
     def split_on_zero(self, remove_none_dps=False) -> tuple[DiffractionExperiment | None, DiffractionExperiment | None]:
         """
@@ -1438,8 +1496,8 @@ class DiffractionExperiment:
                 diffpats_p.append(dpp)
                 diffpats_n.append(dpn)
 
-        dep = DiffractionExperiment(diffpats=diffpats_p) if diffpats_p else None
-        den = DiffractionExperiment(diffpats=diffpats_n) if diffpats_n else None
+        dep = DiffractionExperiment(diffpats=diffpats_p, meta=copy.deepcopy(self.meta)) if diffpats_p else None
+        den = DiffractionExperiment(diffpats=diffpats_n, meta=copy.deepcopy(self.meta)) if diffpats_n else None
         return dep, den
 
 
@@ -1510,12 +1568,13 @@ class Read:
     """
 
     @staticmethod
-    def read(filename: str) -> List[DiffractionDataPoint]:
+    def read(filename: str) -> List[DiffractionDataPoint | DiffractionPattern]:
         """
         This guesses which read method to try based on filename.
         :param filename: string representing the contents of the file
         :return: a list of XRayDataPoints
         """
+        filename = filename.lower()
         if filename.endswith(".xye"):
             return Read.xye(filename)
         elif filename.endswith(".xy"):
@@ -1524,6 +1583,8 @@ class Read:
             return Read.dat(filename)
         elif filename.endswith(".xrdml"):
             return Read.xrdml(filename)
+        elif filename.endswith(".ttx"):
+            return Read.ttx(filename)
         # getting here means that I don't know how to read the file
         raise ValueError(f"I don't know how to read {filename}.")
 
@@ -1533,7 +1594,7 @@ class Read:
         increasing = True
         # print(f"Now reading {filename}.")
         with open(filename) as f:
-            print(f"Reading from {os.path.abspath(filename)}.")
+            print(f"Reading from {shrink_path_for_display(filename)}.")
             for line in f:
                 s = line.split()  # splits on whitespace
                 try:
@@ -1567,7 +1628,7 @@ class Read:
         lst = []
         # print(f"Now reading {filename}.")
         with open(filename) as f:
-            print(f"Reading from {os.path.abspath(filename)}.")
+            print(f"Reading from {shrink_path_for_display(filename)}.")
             # this first for is to skip the comments and grab the start step stop values
             for line in f:
                 s = line.split()  # splits on whitespace
@@ -1608,7 +1669,7 @@ class Read:
         lst = []
         # print(f"Now reading {filename}.")
         with open(filename) as f:
-            print(f"Reading from {os.path.abspath(filename)}.")
+            print(f"Reading from {shrink_path_for_display(filename)}.")
 
             document = ET.parse(filename)
             startAngle = document.find(".//{*}xrdMeasurement/{*}scan[1]/{*}dataPoints/{*}positions[1]/{*}startPosition")
@@ -1626,6 +1687,43 @@ class Read:
 
         return lst
 
+    @staticmethod
+    def ttx(filename: str) -> List[DiffractionPattern]:
+        lst = []
+        with open(filename) as f:
+            print(f"Reading from {shrink_path_for_display(filename)}.")
+            line = f.readline()  # first line
+            line = f.readline()  # second line
+            line = f.readline()  # third line - the names of the dictionary keys
+            meta_keys = line.strip().split()
+            meta_keys += ["Scan", "Acq_time"]
+            line = f.readline()  # the first ***** line
+
+            while True:
+                line = f.readline()  # scan number
+                if not line:
+                    break
+                scan_num = line.strip().split()[-1]
+                line = f.readline()  # motor positions
+                motor_posns = line.strip().split()
+                line = f.readline()  # acquisition time
+                acq_time = line.strip().split()[-1]
+
+                motor_posns += [scan_num, acq_time]
+                meta = dict(zip(meta_keys, motor_posns))
+                ddps = []
+                while not line.startswith("***"):
+                    line = f.readline()
+                    if not line:
+                        break
+                    if line.startswith("***"):
+                        continue
+                    angle, intensity = line.strip().split(",")
+                    ddp = DiffractionDataPoint(float(angle), float(intensity))
+                    ddps.append(ddp)
+                lst.append(DiffractionPattern(diffpat=ddps, filename=filename, meta=meta))
+        return lst
+
 
 # --------------------------------------------------------------------------------------------------
 # --------------------------------------------------------------------------------------------------
@@ -1637,22 +1735,26 @@ class Write:
     """
 
     @staticmethod
-    def _get_padding(dp: DiffractionPattern) -> tuple(int, int, int):
-        x_pad = int(math.log10(max(dp.xs))) + 2
-        y_pad = int(math.log10(max(dp.ys))) + 2
-        e_pad = max(int(math.log10(max(dp.es))), 0) + 2
+    def _get_padding(dp: DiffractionPattern) -> tuple[int, int, int]:
+        x_val = max(math.fabs(max(dp.xs)), math.fabs(min(dp.xs)))
+        y_val = max(math.fabs(max(dp.ys)), math.fabs(min(dp.ys)))
+        e_val = max(math.fabs(max(dp.es)), math.fabs(min(dp.es)))
+
+        x_pad = int(math.log10(x_val)) + 3
+        y_pad = int(math.log10(y_val)) + 3
+        e_pad = max(int(math.log10(e_val)), 0) + 2
         return x_pad, y_pad, e_pad
 
     @staticmethod
     def _get_formatted(d: DiffractionDataPoint, x_dp: int, y_dp: int, e_dp: int,
-                       x_pad: int, y_pad: int, e_pad: int) -> tuple(str, str, str):
+                       x_pad: int, y_pad: int, e_pad: int) -> tuple[str, str, str]:
         a = f"{d.x:{1 + x_pad + x_dp}.{x_dp}f}"
         i = f"{d.y:{1 + y_pad + y_dp}.{y_dp}f}"
         e = f"{d.e:{1 + e_pad + e_dp}.{e_dp}f}"
         return a, i, e
 
     @staticmethod
-    def xye(dp: DiffractionPattern, filename: str,
+    def xye(dp: DiffractionPattern, filenamebase: str,
             dp_angle: int = 5, dp_intensity: int = 3, dp_error: int = 3, is_xy: bool = False) -> None:
         """
         To write a nice XYE file to a file.
@@ -1674,11 +1776,12 @@ class Write:
         :param is_xy:
         :return:
         """
-        print(f"Writing to {os.path.abspath(filename)}.")
+        ext = ".xy" if is_xy else ".xye"
+        filename = filenamebase + ext
         with open(filename, "w") as f:
-            diffpat = dp.diffpat
+            print(f"Writing to {shrink_path_for_display(filename)}.")
             x_pad, y_pad, e_pad = Write._get_padding(dp)
-
+            diffpat = dp.diffpat
             for d in diffpat:
                 a, i, e = Write._get_formatted(d, dp_angle, dp_intensity, dp_error, x_pad, y_pad, e_pad)
                 if is_xy:
@@ -1687,13 +1790,32 @@ class Write:
                     f.write(f"{a}{i}{e}\n")
 
     @staticmethod
-    def xy(dp: DiffractionPattern, filename: str, dp_angle: int = 5, dp_intensity: int = 3, dp_error: int = 3) -> None:
-        Write.xye(dp, filename, dp_angle, dp_intensity, dp_error, is_xy=True)
+    def xyes(de: DiffractionExperiment, filenamebase: Union[str, List],
+             dp_angle: int = 5, dp_intensity: int = 3, dp_error: int = 3, is_xy: bool = False):
+        if isinstance(filenamebase, str):
+            pad = int(math.log10(len(de))) + 1
+            for i, dp in enumerate(de.diffpats):
+                Write.xye(dp, f'{filenamebase}_{i:0{pad}}', dp_angle=dp_angle, dp_intensity=dp_intensity, dp_error=dp_error, is_xy=is_xy)
+        elif isinstance(filenamebase, list):
+            if len(filenamebase) != len(de):
+                raise ValueError(f"The number of filenames ({len(filenamebase)}) is not the same as the number of diffraction patterns ({len(de)}).")
+
+            for i, dp in enumerate(de.diffpats):
+                Write.xye(dp, f'{filenamebase[i]}', dp_angle=dp_angle, dp_intensity=dp_intensity, dp_error=dp_error, is_xy=is_xy)
 
     @staticmethod
-    def cif(dp: DiffractionPattern, filename: str, dp_angle: int = 5,
-            data_block: str = "pdiffutils", x_type: str = "_pd_meas_2theta_scan", y_type: str = "_pd_meas_intensity_total",
-            other_dataitems: str = None) -> None:
+    def xy(dp: DiffractionPattern, filenamebase: str, dp_angle: int = 5, dp_intensity: int = 3, dp_error: int = 3) -> None:
+        Write.xye(dp, filenamebase, dp_angle, dp_intensity, dp_error, is_xy=True)
+
+    @staticmethod
+    def xys(de: DiffractionExperiment, filenamebase: str, dp_angle: int = 5, dp_intensity: int = 3, dp_error: int = 3) -> None:
+        Write.xyes(de, filenamebase, dp_angle, dp_intensity, dp_error, is_xy=True)
+
+    @staticmethod
+    def cif(dp: DiffractionPattern, filenamebase: str, dp_angle: int = 5,
+            data_block: str = "pdiffutils",
+            x_type: str = "_pd_meas_2theta_scan", y_type: str = "_pd_proc_intensity_total",
+            other_dataitems: str = "", append:bool = False) -> None:
         diffpat = dp.diffpat
         x_pad, _, _ = Write._get_padding(dp)
         ys = [val_err_str(y, e) for y, e in zip(dp.ys, dp.es)]
@@ -1701,9 +1823,13 @@ class Write:
         for y in ys:
             max_len_y = max(max_len_y, len(y))
 
-        print(f"Writing to {os.path.abspath(filename)}.")
-        with open(filename, "w") as f:
-            f.write(f"data_{data_block}\n")
+        filename = filenamebase + ".cif"
+
+        write_type = "a" if append else "w"
+        with open(filename, write_type) as f:
+            print(f"Writing to {shrink_path_for_display(filename)}.")
+            f.write(f"\ndata_{data_block}\n")
+            f.write(f"_pd_block_id\t{data_block}\n")
             if other_dataitems:
                 f.write(other_dataitems)
             f.write("\tloop_\n")
@@ -1713,37 +1839,70 @@ class Write:
                 x, _, _ = Write._get_formatted(d, dp_angle, 3, 3, x_pad, 3, 3)
                 f.write(f"\t{x} {y.rjust(max_len_y, ' ')}\n")
 
+    @staticmethod
+    def cifs(de: DiffractionExperiment, filenamebase: str, dp_angle: int = 5,
+             data_blocks: str | List[str] = "",
+             x_type: str = "_pd_meas_2theta_scan", y_type: str = "_pd_meas_intensity_total",
+             other_dataitems: str | List[str] = "") -> None:
+        pad = int(math.log10(len(de))) + 1
+        if not other_dataitems:
+            other_dataitems = [""] * len(de)
+
+        if not data_blocks:
+            data_blocks = [dp.filename for dp in de.diffpats]
+
+        if isinstance(data_blocks, list) and len(data_blocks) != len(de):
+            raise ValueError(f"You gave {len(data_blocks)} data block names. You need {len(de)}.")
+        if isinstance(other_dataitems, list) and len(other_dataitems) != len(de):
+            raise ValueError(f"You gave {len(other_dataitems)} other data items. You need {len(de)}.")
+        if isinstance(other_dataitems, str):
+            other_dataitems = [other_dataitems] * len(de)
+        if isinstance(data_blocks, str):
+            data_blocks = [f"{data_blocks}_{i:0{pad}}" for i in range(len(de))]
+
+        for dp, db, other in zip(de.diffpats, data_blocks, other_dataitems):
+            Write.cif(dp, filenamebase, dp_angle=dp_angle, data_block=db,
+                      x_type=x_type, y_type=y_type, other_dataitems=other, append=True)
+
+    @staticmethod
+    def xyz(de: DiffractionExperiment, filenamebase: str,
+            dp_angle: int = 5, dp_intensity: int = 3, ys: list = None) -> None:
+        """
+        Write an XYZ file of a DiffractionExperiment. If you want a different y-ordinate, you need to create
+        that yourself and have it all pre-formatted, and pass it in to the function.
+        :param de: the DiffractionExperiment to write to file
+        :param filenamebase: the name of the file to write to, without any extension
+        :param dp_angle: how many decimal places to write for the x-ordinate
+        :param dp_intensity: how many decimal place to write for the intensity (z-ordinate)
+        :param ys: preformatted list of strings ready to to put directly into the output. or None, if you just want the patterns numbered
+        :return: None.
+        """
+        if ys and len(ys) != len(de):
+            raise ValueError(f"You gave {len(ys)} y values. You need {len(de)}.")
+        filename = filenamebase + ".xyz"
+
+        x_pad = 0
+        i_pad = 0
+        for dp in de.diffpats:
+            dp_x_pad, dp_i_pad, _ = Write._get_padding(dp)
+            x_pad = max(x_pad, dp_x_pad)
+            i_pad = max(i_pad, dp_i_pad)
+        y_pad = len(str(len(de))) + 2
+
+        if not ys:  # then just number according to ordering in the de.
+            ys = [str(i + 1).rjust(y_pad) for i in range(len(de))]
+
+        with open(filename, "w") as f:
+            print(f"Writing to {shrink_path_for_display(filename)}.")
+            for dp, y in zip(de.diffpats, ys):
+                for ddp in dp.diffpat:
+                    a, i, _ = Write._get_formatted(ddp, dp_angle, dp_intensity, 3, x_pad, i_pad, 3)
+                    f.write(f"{a}{y}{i}\n")
+
 
 def main():
-    diffpat1 = [DiffractionDataPoint(5.00, 4.1),
-                DiffractionDataPoint(5.01, 2.1),
-                DiffractionDataPoint(5.02, 3.1),
-                DiffractionDataPoint(5.03, 4.1),
-                DiffractionDataPoint(5.04, 6.1),
-                DiffractionDataPoint(5.05, 5.1)]
+    pass
 
-    diffpat2 = [DiffractionDataPoint(-5.05, 5.1),
-                DiffractionDataPoint(-5.04, 6.1),
-                DiffractionDataPoint(-5.03, 4.1),
-                DiffractionDataPoint(-5.02, 3.1),
-                DiffractionDataPoint(-5.01, 2.1),
-                DiffractionDataPoint(-5.00, 4.1)]
-
-    diffpat = copy.deepcopy(diffpat2 + diffpat1)
-    dp0a = DiffractionPattern(diffpat=diffpat)
-    dp0b = copy.deepcopy(dp0a)
-    de0 = DiffractionExperiment(diffpats=[dp0a,dp0b])
-
-    de1 = DiffractionExperiment(diffpats=[DiffractionPattern(diffpat=copy.deepcopy(diffpat1))])
-    de2 = DiffractionExperiment(diffpats=[DiffractionPattern(diffpat=copy.deepcopy(diffpat2))])
-    de2.negate()
-    de2.reverse()
-
-    de0 = DiffractionExperiment(diffpats=[DiffractionPattern(diffpat=copy.deepcopy(diffpat2))])
-    dep, den = de0.split_on_zero()
-
-    print(f"{dep=}")
-    print(f"{den=}")
 
 if __name__ == "__main__":
     main()
